


/* Zip Code Program, with multi-threading - full example */
/* NOTE: this program will run for a LONG time. Likely > 7 hours */
/* Author: Andrew Clapson */
/* Date: March 2016*/

%*Define the THREAD  library and include the threading program;  
libname THREAD 'C:\SAS_TempStorage\THREAD'; 
%include 'C:\SAS_TempStorage\ThreadingMaster.sas'; 


%*Create the cross product of ALL US zip codes;  
proc SQL; 
	CREATE TABLE THREAD.ZipCode_1 AS 
		SELECT left(put(A.Zip, Z5.)) AS Zip_A LENGTH = 5, left(put(B.Zip, Z5.)) AS Zip_B LENGTH = 5
			FROM SASHELP.Zipcode A, SASHELP.Zipcode B 
				WHERE (A.Zip NE B.Zip); 
QUIT; 


/**
* multiThreadDataStep macro
* @author Ian J. Ghent
* @param numberOfThreads Number of threads to divide the data step into
* @param inputDatasetName The input dataset which will be used for the data step
* @param partitionMethod Select HORIZONTAL, INTERLEAVED, OR INDEXED (custom needs)
* @param minionThreadCodeLocation Location of the minion thread code
* @param minionThreadLogsLocation Location where the minion thread logs are to go
* @param horizontalLeadObs In horizontal partitioning, number of obs to lead, so that
* some inter-row dependent functions such as LAG() can be
* used
* @return outputDatasetName The processes dataset with the desired name as specified
*/
%*Call the threading code with the parameter values; 
%multiThreadDataStep(4, THREAD.ZipCode_1, HORIZONTAL, C:\SAS_TempStorage\ThreadingMinion.sas, 
 C:\SAS_TempStorage, , WORK.ZipCode_2); 


%*Now sort by Zip code, and descending distance to other zip codes; 
proc sort data = WORK.ZipCode_2;
	by Zip_A Distance Zip_B; 
run; 


%*Because we only want the *3* closest zip codes (and the data is sorted) we only keep the first *3* rows for each Zip_A; 
data WORK.ZipCode_3;
set WORK.ZipCode_2; 
	by Zip_A Distance; 
		if (first.Zip_A = 1) then do;
			numZipCodes = 1;
		end;
			else do;  
				numZipCodes + 1; 
			end; 
		if numZipCodes <= 3 then output; 
run; 


%*Then transpose the dataset by Zip_A (our key) to give us the three closest ZipCodes as columns; 
proc transpose data = WORK.ZipCode_3 
			   out = WORK.ZipCode_4 (drop = _NAME_) prefix = ClosestZip_;
	by Zip_A;
		ID numZipCodes; 
			var Zip_B; 
run; 


%*Now! We have EVERY distance ready, ranked, so enter an example for the zip codes that we might be matching.;
data WORK.ZipsToMatch_1;
input Zip_A $5.;
datalines;
64486
66939
69352
11111
;
run; 


%*Merge by zip codes and keep all those that match; 
proc sort data = WORK.ZipsToMatch_1; 
	by Zip_A; 
run; 

data WORK.MatchedCodes; 
merge WORK.ZipCode_4 (in = a) WORK.ZipsToMatch_1 (in = b);
	if a & b then output; 
run; 


%*Hash by zip codes and keep all those that match; 
data WORK.MatchedCodes_2; 
	if 0 then set WORK.ZipsToMatch_1 WORK.ZipCode_4; %*Load the variable properties (length, formats, etc.); 

	set WORK.ZipsToMatch_1; 

	if _n_ = 1 then do; %*Only set up one Hash table; 
		declare Hash findZipCode(dataset:'WORK.ZipCode_4'); 
			findZipCode.defineKey('Zip_A'); 
 			findZipCode.defineData('ClosestZip_1', 'ClosestZip_2', 'ClosestZip_3');
		findZipCode.defineDone(); 
	end; 

	if findZipCode.find(key:Zip_A) = 0 then output; %*Now, take each row of the 'to be matched' dataset, and find its key match; 
run; 
