

/* Zip Code Program - subsetting example */
/* Author: Andrew Clapson */
/* Date: March 2016*/

%*Create the cross product of a subset of the zip codes (just to make this example quicker); 
proc SQL; 
	CREATE TABLE ZipCode_1 AS 
		SELECT left(put(A.Zip, Z5.)) AS Zip_A LENGTH = 5, left(put(B.Zip, Z5.)) AS Zip_B LENGTH = 5
			FROM SASHELP.Zipcode A, SASHELP.Zipcode B 
				WHERE (substr(left(put(A.Zip, Z5.)), 1, 2) = '05') AND (substr(left(put(B.Zip, Z5.)), 1, 2) = '05')
				AND (A.Zip NE B.Zip); 
QUIT; 


%*Calculate the distance between each zip code and every other one; 
data WORK.ZipCode_2; 
set WORK.ZipCode_1; 
	Distance = zipcitydistance(Zip_A, Zip_B);  
run; 


%*Now sort by Zip code, and descending distance to other zip codes; 
proc sort data = WORK.ZipCode_2;
	by Zip_A Distance Zip_B; 
run; 


%*Because we only want the *3* closest zip codes (and the data is sorted) we only keep the first *3* rows for each Zip_A; 
data WORK.ZipCode_3;
set WORK.ZipCode_2; 
	by Zip_A Distance; 
		if (first.Zip_A = 1) then do;
			numZipCodes = 1;
		end;
			else do;  
				numZipCodes + 1; 
			end; 
		if numZipCodes <= 3 then output; 
run; 


%*Then transpose the dataset by Zip_A (our key) to give us the three closest ZipCodes as columns; 
proc transpose data = WORK.ZipCode_3 
			   out = WORK.ZipCode_4 (drop = _NAME_) prefix = ClosestZip_;
	by Zip_A;
		ID numZipCodes; 
			var Zip_B; 
run; 


%*Now! We have EVERY distance ready, ranked, so enter an example for the zip codes that we might be matching.;
data WORK.ZipsToMatch_1;
input Zip_A $5.;
datalines;
64486
66939
69352
11111
;
run; 


%*Merge by zip codes and keep all those that match; 
proc sort data = WORK.ZipsToMatch_1; 
	by Zip_A; 
run; 

data WORK.MatchedCodes; 
merge WORK.ZipCode_4 (in = a) WORK.ZipsToMatch_1 (in = b);
	if a & b then output; 
run; 


