


/**
* Multi-threaded data steps - Minion Thread Code
* <P>
* This code is not meant to be stand alone. It is meant to be run in batch, via
* a call from a master thread. Insert your normal data step code below the
* "INSERT YOUR NORMAL DATA STEP CODE HERE" comment.
* </P>
* @author Ian J. Ghent
* @created 2009-08-23
* @modified 2010-02-09 by Ian J. Ghent
* @version 1.0
*/
/**
* Thread macro
* @author Ian J. Ghent
*/

%macro thread(); 

	%let threadId = &sysparm; 

	* If the host OS supports processor affinity commands (ie. Solaris pbind, Linux taskset) and you ;
	* want to make use of that, this would be where to put those calls via a systask or x command. ;
	* You can reference this process id using the SYSJOBID system macro variable. This should only be ;
	* done, if you know in advance, what the processor load for other tasks on the system will be. ;

	%*Assign any needed libraries;
	libname THREAD 'C:\SAS_TempStorage\THREAD'; 

	%*Change the name of the dataset to reflect the status ;
	proc datasets library = THREAD nolist nodetails;
		change thread_not_started&threadId. = thread_running&threadId.;
	quit; 

	%*Dump the information dataset to macro variables and update the start timestamp ;
	data THREAD.thread_running&threadId.;
	set THREAD.thread_running&threadId.;
		call symputx('numberOfThreads', number_of_threads);
		call symputx('datasetName', dataset_name);
		call symputx('partitionMethod', partition_method);
		call symputx('firstObs', first_obs);
		call symputx('lastObs', last_obs);
		call symputx('whereClause', where_clause);
		start_datetime = datetime();
	run; 

	%*start the main data step; 
	data THREAD.thread_part&threadId.;
	set &datasetName. 
		%* depending upon the partition method, only read selected observations ;
		%if &partitionMethod. = HORIZONTAL %then %do;
			(firstobs = &firstObs. obs = &lastObs.); 
		%end;
		%else %if &partitionMethod. = INTERLEAVED %then %do;
			; 
			%if &threadId. = &numberOfThreads. %then %let modValue = 0;
				%else %let modValue = &threadId.; 
			if mod(_n_, &numberOfThreads.) = &modValue.;
			* provided in case the dataset needs to be sorted in the original order ;
			record_number = _n_; 
		%end; 
			%else %if &partitionMethod. = INDEXED %then %do;
				(where = (&whereClause.));
			%end; 

		**** INSERT NORMAL DATA STEP CODE HERE ****;
		Distance = zipcitydistance(Zip_A, Zip_B); 
	run; 

	%*Record the timestamp when the data step finished execution, and calculate the run time ;
	data THREAD.thread_running&threadId.; 
	set THREAD.thread_running&threadId.;
		finish_datetime = datetime();
		run_time_minutes = intck('MINUTE', finish_datetime, start_datetime);
	run; 

	%*Change the information dataset name to reflect the status ;
	proc datasets library = THREAD nolist nodetails;
		change thread_running&threadId. = thread_completed&threadId.;
	quit; 

%mend thread;

%thread() 
